package GII::Event::Expose;

=head1 NAME

GII::Event::Expose - Perl class encapsulating gii_expose_event structures.

=head1 SYNOPSIS

  use GII::Event::Expose;

  $ev = new GII::Event::Expose;

  $x = $ev->x;
  $ev->x($x);

  $y = $ev->y;
  $ev->y($y);

  $w = $ev->w;
  $ev->w($w);

  $h = $ev->h;
  $ev->h($h);

  ($x, $y) = $ev->xy;
  $coord->coord($ev->xy);
  $ev->xy($x, $y);
  $ev->xy($coord);

  ($w, $h) = $ev->wh;
  $coord->coord($ev->wh);
  $ev->wh($w, $h);
  $ev->wh($coord);

  ($x, $y, $w, $h) = $ev->exposure;
  $ev->exposure($x, $y, $w, $h);
  $ev->exposure($top_left_corner_coord, $size_coord);

=head1 DESCRIPTION

The GII::Event::Expose subclass provides simple methods to access
the extra members of a LibGII gii_expose_event structure with cannot
be accessed via the methods in the class GII::Event.

=cut

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use GII::Event;

require DynaLoader;
require AutoLoader;

@ISA = qw(AutoLoader DynaLoader GII::Event);
# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

$VERSION = '0.01';

bootstrap GII::Event::Expose $VERSION;

=head1 Methods

The extra methods and differences between GII::Event methods and the
ones provided in the GII::Event::Expose class are as follows:

=head2 new

The GII::Event::Expose constructor takes all the same arguments
as the GII::Event constructor, and treats them the same with the
exception that it does not do any automagical subclassing.  All
calls to this constructor will either fail or allocate/reuse
a gii_expose_event structure.

Since this event is generated by the environment, this constructor
will be used only very rarely for debugging purposes.

=cut

sub new {
    my($class) = shift();
    my($type) = shift();
    my($this) = {};
    bless $this, $class;
    my($obj) = shift();
    if ($obj) {
	$this->{obj} = $obj;
	$this->set_type($obj, $type) if defined($type);
	$this->set_size($obj, $this->default_size) if defined($type);
	return($this);
    }
    $this->{obj} = $this->_alloc;
    $this->set_size($this->{obj}, $this->default_size);
    $this->set_type($this->{obj}, $type);
    return($this);
}

=head2 x

Without an argument, returns the x coordinate of the top left 
corner of the area which needs to be redrawn as a result of the
exposure event.  With an argument, sets said value, returning
the value which was set.

=cut

sub x {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_x($this->{obj}, shift());
    }
    return $this->get_x($this->{obj});
}

=head2 y

Without an argument, returns the y coordinate of the top left 
corner of the area which needs to be redrawn as a result of the
exposure event.  With an argument, sets said value, returning
the value which was set.

=cut

sub y {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_y($this->{obj}, shift());
    }
    return $this->get_y($this->{obj});
}

=head2 w

Without an argument, returns the width of the area which needs to 
be redrawn as a result of the exposure event.  With an argument, 
sets said value, returning the value which was set.

=cut

sub w {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_w($this->{obj}, shift());
    }
    return $this->get_w($this->{obj});
}

=head2 w

Without an argument, returns the height of the area which needs to 
be redrawn as a result of the exposure event.  With an argument, 
sets said value, returning the value which was set.

=cut

sub h {
    my($this) = shift();
    if (@_ + 0) {
	return $this->set_h($this->{obj}, shift());
    }
    return $this->get_h($this->{obj});
}

=head2 xy

Without an argument, returns the coordinates of the top left 
corner of the area which needs to be redrawn as a result of the
exposure event.  With an argument, sets said values, returning
the values which were set.

=cut

sub xy {
    my($this) = shift();
    if (@_ + 0) {
	my($arg) = shift();
	return ($this->set_x($this->{obj}, $arg->x),
		$this->set_y($this->{obj}, $arg->y))
	    if ref($arg) and UNIVERSAL::isa($arg, "GGI::Coord");
	return ($this->set_x($this->{obj}, $arg),
		$this->set_y($this->{obj}, shift()));
    }
    return ($this->get_x($this->{obj}),$this->get_y($this->{obj}));
}


=head2 wh

Without an argument, returns the width and height of the area which 
needs to be redrawn as a result of the exposure event.  With an argument, 
sets said values, returning the values which were set.

=cut

sub wh {
    my($this) = shift();
    if (@_ + 0) {
	my($arg) = shift();
	return ($this->set_w($this->{obj}, $arg->x),
		$this->set_h($this->{obj}, $arg->y))
	    if ref($arg) and UNIVERSAL::isa($arg, "GGI::Coord");
	return($this->set_w($this->{obj}, $arg), 
	       $this->set_h($this->{obj}, shift()));
    }
    return ($this->get_w($this->{obj}), $this->get_h($this->{obj}));
}

=head2 exposure

Without an argument, returns the coordinates of the top left 
corner and the width and height of the area which needs to be 
redrawn as a result of the exposure event.  With an argument, 
sets said values, returning the values which were set.

=cut

sub exposure {
    my($this) = shift();
    if (@_ + 0) {
	my($arg) = shift;
	my($x, $y);
	if (ref($arg) and UNIVERSAL::isa($arg, "GGI::Coord")) {
	    $x = $arg->x;
	    $y = $arg->y;
	}
	else {
	    $x = $arg;
	    $y = shift();
	}
	my($arg) = shift();
	return ($this->set_x($this->{obj}, $x),
		$this->set_y($this->{obj}, $y),
		$this->set_w($this->{obj}, $arg->x),
		$this->set_h($this->{obj}, $arg->y))
	    if ref($arg) and UNIVERSAL::isa($arg, "GGI::Coord");
	return ($this->set_x($this->{obj}, $x),
		$this->set_y($this->{obj}, $y),
		$this->set_w($this->{obj}, $arg),
		$this->set_h($this->{obj}, shift()));
    }
    return ($this->get_x($this->{obj}),$this->get_y($this->{obj}),
	    $this->get_w($this->{obj}),$this->get_h($this->{obj}));
}

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::(3pm), perl(1).

=cut
