package GII::Event::Valuator;

=head1 NAME

GII::Event::Valuator - Perl class encapsulating gii_val_event structures.

=head1 SYNOPSIS

   use GII::Event::Valuator;

   $ev = new GII::Event::Valuator;

   $first = $ev->first();
   $ev->first($first);

   $count = $ev->count();
   $ev->count($count);

   $val = $ev->val($idx);
   $ev->val($idx, $val);

   $is_relative = $ev->relative;
   $ev->relative($is_relative);

=head1 DESCRIPTION

The GII::Event::Valuator subclass provides simple methods to access
the extra members of a LibGII gii_val_event structure which cannot
be accessed via the methods in the class GII::Event.

=cut

use strict;
use vars qw($VERSION @ISA);
use GII::Event;

require DynaLoader;
require AutoLoader;

@ISA = qw(AutoLoader DynaLoader GII::Event);
$VERSION = '0.01';

bootstrap GII::Event::Valuator $VERSION;

=head1 Methods

The extra methods and differences between GII::Event methods and the
ones provided in the GII::Event::PButton class are as follows:

=head2 new

The GII::Event::Valuator constructor takes all the same arguments
as the GII::Event constructor, and treats them the same with the
exception that it does not do any automagical subclassing.  All
calls to this constructor will either fail or allocate/reuse
a gii_val_event structure.

Since this event is generated by the environment, this constructor
will be used only very rarely for debugging purposes.

=cut

sub new {
    my($class) = shift();
    my($type) = shift();
    my($this) = {};
    bless $this, $class;
    my($obj) = shift();
    if ($obj) {
        $this->{obj} = $obj;
        $this->set_type($obj, $type) if defined($type);
        $this->set_size($obj, $this->default_size) if defined($type);
        return($this);
    }
    $this->{obj} = $this->_alloc;
    $this->set_size($this->{obj}, $this->default_size);
    $this->set_type($this->{obj}, $type);
    return($this);
}

=head2 relative

Without an argument, returns 1 if the valuator event is of type
evValRelative, 0 if the valuator event is of type evValAbsolute,
and undef if the type is not a valid valuator event type.  With
an argument, sets the valuator event type to evValRelative if 
the argument is true in a boolean context, else sets it to 
evValAbsolute, and returns the value which was set.

=cut

sub relative {
    my($this) = shift();
    if (@_ + 0) {
	if (shift()) {
	    $this->set_type($this->{obj}, &evValRelative);
	    return &evValRelative;
	}
	else {
	    $this->set_type($this->{obj}, &evValAbsolute);
            return &evValAbsolute;
	}
    }
    return 1 if $this->get_type($this->{obj}) == &evValRelative;
    return 0 if $this->get_type($this->{obj}) == &evValAbsolute;
    return undef;
}

=head2 first

Without an argument, returns the index of the first valuator
reported.  Valuators are reported in maximum sized chunks of
32 values, so valuator sources with more values will send 
multiple events each reporting different values.  With an 
argument, sets said value, returning the value which was set.

=cut

sub first {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_first($this->{obj},shift());
    }
    return $this->get_first($this->{obj});
}

=head2 count

Without an argument, returns the number of values contained
in the event (maximum 32).  With an argument, sets said value, 
returning the value which was set.

=cut

sub count {
    my($this) = shift();
    if (@_ + 0) {
        return $this->set_count($this->{obj},shift());
    }
    return $this->get_count($this->{obj});
}

=head2 val

The first argument is the index of the value you wish to 
retrieve or set.  This index must be greater than $ev->first,
and not more than $ev->first + count - 1.  A second argument
will cause the value to be set, returning the value which was
set, or undef if there was an error.

=cut

sub val {
    my($this) = shift();
    my($idx) = shift();
    if (@_ + 0) {
        return $this->set_val($this->{obj},$idx, shift());
    }
    return $this->get_val($this->{obj},$idx);
}

1;
__END__

=head1 AUTHOR

Brian S. Julin <bri@calyx.com>

=head1 SEE ALSO

GII::Event(3pm), GII::Event::Command::Valuator(3pm), GII::(3pm), perl(1).

=cut
